Class: Ask::Agent::ApprovalQueue::Action

Inherits:
Data
  • Object
show all
Defined in:
lib/ask/agent/approval_queue.rb

Overview

One queued action.

Instance Attribute Summary collapse

Instance Attribute Details

#argsHash (readonly)

Returns arguments to pass to the tool when applied.

Returns:

  • (Hash)

    arguments to pass to the tool when applied



44
45
# File 'lib/ask/agent/approval_queue.rb', line 44

Action = Data.define(:id, :tool_call_id, :tool_name, :args,
:auto_approvable, :status, :submitted_at, :message)

#auto_approvableBoolean (readonly)

Returns per-action verdict (tool's declaration).

Returns:

  • (Boolean)

    per-action verdict (tool's declaration)



44
45
# File 'lib/ask/agent/approval_queue.rb', line 44

Action = Data.define(:id, :tool_call_id, :tool_name, :args,
:auto_approvable, :status, :submitted_at, :message)

#idInteger (readonly)

Returns sequential id assigned by the queue.

Returns:

  • (Integer)

    sequential id assigned by the queue



44
45
# File 'lib/ask/agent/approval_queue.rb', line 44

Action = Data.define(:id, :tool_call_id, :tool_name, :args,
:auto_approvable, :status, :submitted_at, :message)

#messageString? (readonly)

Returns human-readable description of the action.

Returns:

  • (String, nil)

    human-readable description of the action



44
45
# File 'lib/ask/agent/approval_queue.rb', line 44

Action = Data.define(:id, :tool_call_id, :tool_name, :args,
:auto_approvable, :status, :submitted_at, :message)

#statusSymbol (readonly)

Returns :pending, :applying, :approved, :rejected.

Returns:

  • (Symbol)

    :pending, :applying, :approved, :rejected



44
45
# File 'lib/ask/agent/approval_queue.rb', line 44

Action = Data.define(:id, :tool_call_id, :tool_name, :args,
:auto_approvable, :status, :submitted_at, :message)

#submitted_atTime (readonly)

Returns:

  • (Time)


44
45
# File 'lib/ask/agent/approval_queue.rb', line 44

Action = Data.define(:id, :tool_call_id, :tool_name, :args,
:auto_approvable, :status, :submitted_at, :message)

#tool_call_idString (readonly)

Returns the original tool call id from the LLM.

Returns:

  • (String)

    the original tool call id from the LLM



44
45
# File 'lib/ask/agent/approval_queue.rb', line 44

Action = Data.define(:id, :tool_call_id, :tool_name, :args,
:auto_approvable, :status, :submitted_at, :message)

#tool_nameString (readonly)

Returns tool name.

Returns:

  • (String)

    tool name



44
45
# File 'lib/ask/agent/approval_queue.rb', line 44

Action = Data.define(:id, :tool_call_id, :tool_name, :args,
:auto_approvable, :status, :submitted_at, :message)