Class: Ask::Agent::ApprovalQueue::Action
- Inherits:
-
Data
- Object
- Data
- Ask::Agent::ApprovalQueue::Action
- Defined in:
- lib/ask/agent/approval_queue.rb
Overview
One queued action.
Instance Attribute Summary collapse
-
#args ⇒ Hash
readonly
Arguments to pass to the tool when applied.
-
#auto_approvable ⇒ Boolean
readonly
Per-action verdict (tool's declaration).
-
#id ⇒ Integer
readonly
Sequential id assigned by the queue.
-
#message ⇒ String?
readonly
Human-readable description of the action.
-
#status ⇒ Symbol
readonly
:pending, :applying, :approved, :rejected.
- #submitted_at ⇒ Time readonly
-
#tool_call_id ⇒ String
readonly
The original tool call id from the LLM.
-
#tool_name ⇒ String
readonly
Tool name.
Instance Attribute Details
#args ⇒ Hash (readonly)
Returns arguments to pass to the tool when applied.
44 45 |
# File 'lib/ask/agent/approval_queue.rb', line 44 Action = Data.define(:id, :tool_call_id, :tool_name, :args, :auto_approvable, :status, :submitted_at, :message) |
#auto_approvable ⇒ Boolean (readonly)
Returns per-action verdict (tool's declaration).
44 45 |
# File 'lib/ask/agent/approval_queue.rb', line 44 Action = Data.define(:id, :tool_call_id, :tool_name, :args, :auto_approvable, :status, :submitted_at, :message) |
#id ⇒ Integer (readonly)
Returns sequential id assigned by the queue.
44 45 |
# File 'lib/ask/agent/approval_queue.rb', line 44 Action = Data.define(:id, :tool_call_id, :tool_name, :args, :auto_approvable, :status, :submitted_at, :message) |
#message ⇒ String? (readonly)
Returns human-readable description of the action.
44 45 |
# File 'lib/ask/agent/approval_queue.rb', line 44 Action = Data.define(:id, :tool_call_id, :tool_name, :args, :auto_approvable, :status, :submitted_at, :message) |
#status ⇒ Symbol (readonly)
Returns :pending, :applying, :approved, :rejected.
44 45 |
# File 'lib/ask/agent/approval_queue.rb', line 44 Action = Data.define(:id, :tool_call_id, :tool_name, :args, :auto_approvable, :status, :submitted_at, :message) |
#submitted_at ⇒ Time (readonly)
44 45 |
# File 'lib/ask/agent/approval_queue.rb', line 44 Action = Data.define(:id, :tool_call_id, :tool_name, :args, :auto_approvable, :status, :submitted_at, :message) |
#tool_call_id ⇒ String (readonly)
Returns the original tool call id from the LLM.
44 45 |
# File 'lib/ask/agent/approval_queue.rb', line 44 Action = Data.define(:id, :tool_call_id, :tool_name, :args, :auto_approvable, :status, :submitted_at, :message) |
#tool_name ⇒ String (readonly)
Returns tool name.
44 45 |
# File 'lib/ask/agent/approval_queue.rb', line 44 Action = Data.define(:id, :tool_call_id, :tool_name, :args, :auto_approvable, :status, :submitted_at, :message) |